/* $Workfile: B8BOOT.C $       $Revision: 15 $      $Modtime: 4/30/98 10:25a $  */
/*--------------------------------------------------------------------------*\

  Copyright (C) Odin TeleSystems Inc., 1996


  DESCRIPTION

    Configures the DSPs on a number of Balder-8 2.0 boards.

\*--------------------------------------------------------------------------*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>


#include "balder.h"              /* Balder Standard Application Programming Interface */
#include "otsdef.h"
#include "bldsys.h"
#include "appid.h"

#include "dtmffsk.img"


#define DEFAULT_BASE_ADDR    0x280
#define DEFAULT_NO_OF_BOARDS 1
#define DEFAULT_DSP_APP_FILE "dtmffsk.out"


#define BALDER8_BOARD_TYPE  128       /* Unique code for Balder-8 2.0    */
#define BALDER4_BOARD_TYPE  124       /* Unique code for Balder-8 2.0    */



typedef struct envT {
  short         noOfBoards;
  short         boardType;
  unsigned long baseAddr;
  Bool          exitOnError;
  Bool          loadImage;
  Bool          log;
  char          logFname[128+1];
  char          dspProgFname[128+1];
  short         liMax;
} envT;


/*---------------------------------------------------------------------------*\
  PRIVATE DATA
\*---------------------------------------------------------------------------*/
static envT env;

static FILE *logF;


/*---------------------------------------------------------------------------*\
  FUNCTION PROTOTYPES
\*---------------------------------------------------------------------------*/
void printUsage(void);
static void parseCmdLineArgs(int argc, char *argv[], envT *env);
void logf(char *formatStr, ...);
static void logError(int errCode, char *formatStr, ...);
static void myExit(int exitCode);
static Bool installDriver(void);


#if 1
extern BldRc dspCompileApp(
  char *appfile,
  char *entrypt,
  char *outfile
);

#endif



void main(int argc, char *argv[])
/*---------------------------------------------------------------------------*\
  MAIN
\*---------------------------------------------------------------------------*/
{
  // Initialize env
  env.exitOnError = OTS_TRUE;
  env.loadImage   = OTS_TRUE;
  env.log         = OTS_FALSE;
  env.logFname[0] = '\0';
  env.baseAddr    = DEFAULT_BASE_ADDR;
  env.noOfBoards  = DEFAULT_NO_OF_BOARDS;
  env.liMax = env.noOfBoards * 8;
  strcpy(env.dspProgFname, DEFAULT_DSP_APP_FILE);

  printf("B8BOOT (%s), %s, Copyright (c) Odin TeleSystems Inc. 1998\n", APP_IDENT, __DATE__);
  printf("Using driver: %s\n", bldIdentDriver());

  parseCmdLineArgs(argc, argv, &env);

  env.boardType = BALDER8_BOARD_TYPE;
  if (env.liMax == 4) {
    env.boardType = BALDER4_BOARD_TYPE;
  }

  env.liMax--;

  if (env.log) {
    if ((logF = fopen(env.logFname, "at")) == NULL) {
      logf("Couldn't open logfile %s\n", env.logFname);
      exit(2);
    }
  }


  logf("Using BaseAddr         = 0x%.lX\n", env.baseAddr);
  logf("# of DSPs to be booted = %d\n", env.liMax+1);
  if (env.log) {
    logf("Log file               = %s\n", env.logFname);
  }
  if (env.loadImage) {
    logf("Loading from DSP Image (DTMFFSK.IMG)\n");
  } else {
    logf("Loading from DSP Application File = %s\n", env.dspProgFname);
  }

  if (!installDriver()) {
    myExit(1);
  }

  printf("B8BOOT complete.\n");

  myExit(0);
}



/*---------------------------------------------------- Function Description -*\
  DESCRIPTION

    Sets up the driver.

  RETURNS
    TRUE   // No error(s)
    FALSE  // Error(s)
\*---------------------------------------------------------------------------*/
static Bool installDriver(void)
{
  BldRc rc; // Return code from BLD functions
  short liNo;

  if ((rc = bldConstructDriver(env.boardType, env.baseAddr, env.noOfBoards, BLD_NO_IRQ)) != BLD_SUCCESS) {
    logf("Error #%d: %s\n", (int) rc, bldGetErrMsg(rc));
    myExit(rc);
  }

  if ((rc = bldInitDriver()) != BLD_SUCCESS) {
    logf("Error #%d: %s\n", (int) rc, bldGetErrMsg(rc));
    myExit(rc);
  }


  for (liNo = 0; liNo <= env.liMax; liNo++) {
    sysDspSetResetPin(liNo, 0);  // Bring DSP into RESET (to bring HINT inactive)
  }


  for (liNo = 0; liNo <= env.liMax; liNo++) {
    if (env.loadImage) {
      logf("Loading DSP Image to DSP#%d \n", liNo);
      if ((rc = bldConfigureDspImage(liNo, (unsigned short *)dspProgImage, DSP_PROG_IMAGE_LEN, DSP_PROG_ENTRY_ADDR, LT_ALAW)) != BLD_SUCCESS) {
        logf("Error: %s\n", bldGetErrMsg(rc));
        myExit(rc);
      }
    } else {
      logf("Loading DSP Application to DSP#%d \n", liNo);
      if ((rc = bldConfigureDsp(liNo, env.dspProgFname, "_c_int00", LT_ALAW)) != BLD_SUCCESS) {
        logf("Error: %s\n", bldGetErrMsg(rc));
        myExit(rc);
      }
    }
  }

  return OTS_TRUE;
}

/*---------------------------------------------------------------------------*\

  SYNOPSIS


  RETURNS


\*---------------------------------------------------------------------------*/
void printUsage()
{
  logf("Usage: \n");
  logf("  b8boot [A={IMAGE | FILE}] [<PAR>=<xxx> ...]\n");
  logf("                                                      \n");
  logf("    Action Parameters                                 \n");
  logf("      A=IMAGE (load DSP Image program to DSPs, default)\n");
  logf("      A=FILE (load specified DSP program to DSPs      \n");
  logf("                                                      \n");
  logf("    Board Communcation parameters                     \n");
  logf("      B=<No_Of_Boards>  (1, 2, 3, or 4>               \n");
  logf("      IBA=<I/O_Base_Addr>                             \n");
  logf("      F=<DSP_program_file>  (to be used with A=FILE)  \n");
  logf("                                                      \n");
  logf("Examples: \n");
  logf("  b8boot A=FILE IBA=0x280 F=DSPAPP.OUT                \n");
  logf("  b8boot IBA=640                                      \n");
  logf("                                                      \n");
  logf("Default Values:                                       \n");
  logf("  B=1                                                 \n");
  logf("  IBA=0x280                                           \n");
  logf("  A=IMAGE  (the image being DTMFFSK.OUT)              \n");
  logf("                                                      \n");
}

/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Positions a pointer to the value string followin the
    '=' sign. If no equal sign is found, or if the space
    after the '=' sign is empty, then an error message
    is printed.

  RETURNS

    Pointer to the first character of the value string.
    or program exit;

\*---------------------------------------------------------------------------*/
static char *getVal(char *par) {
  while (*par) {
    if (*par == '=') {
      *par ='\0';
      par++;
      if (!*par) {
        printUsage();
        myExit(5);
      }
      return par;
    }
    par++;
  }

  printUsage();

  myExit(5);

  return(NULL);       // To make the compiler happy...
}



static void parseCmdLineArgs(
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    The environment variable is filled out

    Parse the programs command line arguments and calls upon requested
    actions.

    The command line parameters are of the format:

    XXX=yyy

    where XXX is the parameter identification string (case insensitive) and
    the yyy is the value (decimal, hexadecimal or a string).

  RETURNS


\*---------------------------------------------------------------------------*/
  int argc,
  char *argv[],
  envT *env
)
{
  int  i;
  char *par;
  char *value;

  for (i = 1; i < argc; i++) {        // Scan command line arguments
    //printf("argv[%d] = %s\n", i, argv[i]);
    strupr(argv[i]);
    value = getVal(argv[i]);

    par = argv[i];
    if (!strcmp(par, "IBA")) {          // I/O Base Address
      env->baseAddr = strtoul(value, NULL, 0);
    } else if (!strcmp(par, "B")) {   // Board No
      env->noOfBoards = (int) strtol(value, NULL, 0);
    } else if (!strcmp(par, "A")) {   // Action
      if (!stricmp(value, "IMAGE")) {
        env->loadImage = OTS_TRUE;
      } else if (!stricmp(value, "FILE")) {
        env->loadImage = OTS_FALSE;
      } else {
        logf("ERROR: Valid action ( A= ) is IMAGE or FILE\n");
        myExit(7);
      }
    } else if (!strcmp(par, "F")) {   // DSP program file
      strcpy(env->dspProgFname, value);
    } else if (!strcmp(par, "LOG")) {   // Log to file
      env->log = OTS_TRUE;
      strcpy(env->logFname, value);
    } else if (!strcmp(par, "EXIT")) {   // Exit On Error
      if ( !strcmp(value,"YES")) {
        env->exitOnError = OTS_TRUE;
      } else {
        env->exitOnError = OTS_FALSE;
      }
    } else if (!strcmp(par, "N")) {   // DSP program file
      env->liMax = (short) strtoul(value, NULL, 0);
    } else {
      logf("ERROR: Unknown parameter %s\n", argv[i]);
      exit(6);
    }
  }
}





void logf(char *formatStr, ...)
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Logs to log file (if present) and to screen.

\*--------------------------------------------------------------------------*/
{
  va_list argList;
  va_start(argList, formatStr);

  vfprintf(stdout, formatStr, argList);

  if (env.log) {
    vfprintf(logF,   formatStr, argList);
  }

  va_end(argList);

}


static void logError(int errCode, char *formatStr, ...)
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Logs an error message returned from the driver.

\*--------------------------------------------------------------------------*/
{
  va_list argList;
  va_start(argList, formatStr);

  fprintf(stdout, "\nERROR #%d: ", errCode);
  vfprintf(stdout, formatStr, argList);
  fprintf(stdout, "\n");

  if (env.log) {
    fprintf(logF, "\nERROR #%d: ", errCode);
    vfprintf(logF, formatStr, argList);
    fprintf(logF, "\n");
  }

  va_end(argList);

  if (env.exitOnError) {
    myExit(errCode);
  }
}

static void myExit(int exitCode)
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Performs a safe exit from the program.

\*--------------------------------------------------------------------------*/
{
  BldRc rc;
  if ((rc = bldDestructDriver()) != BLD_SUCCESS) {
    logf("Error #%d: %s\n", (int) rc, bldGetErrMsg(rc));
  }

  if (logF) {
    fclose(logF);
  }

  exit(exitCode);
}






