/* $Workfile: BALDER.H $       $Revision: 12 $      $Modtime: 5/08/98 9:56p $  */
/*-------------------------------------------------------- File Description -*\

  Copyright (C) Odin TeleSystems Inc., 1994-1997

  NAME

    BALDER.H

  DESCRIPTION

    Header containing the Application Programming Interface (API) for
    Odin TeleSystems' Balder-8S Rev. 2.0 ISDN S/T interface board
    (Odin Product Number HAA-1003-1-2.0)

    This header should used in conjunction with the Balder driver library,
    Balder8s.lib

  COMPILERS

    Microsoft Visual C++,  Version 5.0
    Borland C++, Version 4.53
    Microsoft C, Version 5.1


  SEE ALSO

    Balder-8S Installation Guide (Odin Document Number 1512-1-HAA-1003-1-2.0)
    Balder-8 2.0 API Reference Guide


  Copyright (C) Odin TeleSystems Inc., 1994-1997

  All rights reserved. No part of this computer program may be reproduced,
  stored in any retrieval system, or transmitted, in any form or by any
  means, electronic, mechanical, photocopying, recording, or otherwise,
  without the prior written permission of Odin TeleSystems Inc., or without
  a permission stipulated in the agreement/contract under which this computer
  program was supplied.

\*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*\
  NAMING CONVENTIONS

    All the types and functions used in this API have a prefix 'Bld' (For
    Balder).

    The function names consist of three parts: the header 'Bld', operation,
    and target. The operation word is a verb or phrase describing the action
    to be taken (e.g., construct, read). The target can either be a physical
    subsystem block (e.g., 'Li' or 'Driver') or a signal processing function
    (e.g., 'Silence Detector' or 'DtmfDetector'). Note that 'Driver' target
    functions effect the whole system, whereas the 'Li' function only affects
    specified Line Interfaces.

  NUMBERING OF LI'S

    The Balder boards should be numbered from 0 to MAX_NO_OF_BOARDS.
    Each Balder board contains 8 Line Interfaces (ISAC-S chips).
    The LIs are numbered consecutively from 0 to 8 * MAX_NO_OF_BOARDS - 1.
    For example, if we have three boards, the numbering is as follows:

        BOARD #         LI #
        --------------------
           0           0-7
        --------------------
           1           8-15
        --------------------
           2           16-23
        --------------------

  NUMBERING OF DIGITAL SIGNAL PROCESSORS

    The 8 DSPs use the same number scheme as the Line Interfaces.

  EXCEPTION HANDLING

    Exception handling is taken care with return codes. Each function
    returns 0 if successful (no errors) or a non-zero value in the case
    of a problem. The return codes can be translated into strings
    containing the error message with the bldGetErrMsg() function.

\*---------------------------------------------------------------------------*/
#ifndef BALDER_H
#define BALDER_H

#include "blddef.h"

#ifdef __cplusplus
  extern "C" {
#endif

/*---------------------------------------------------------------------------*\
  GLOBAL VARIABLES
\*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*\
  FUNCTION DECLARATIONS
\*---------------------------------------------------------------------------*/

#ifndef B8_LITE
char *bldIdentDriver(
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Returns a pointer to an identification string for the driver.
    Useful for situations where the driver and the applications are
    not statically linked, and an application may want to query for
    the revision or name of a currently dynamically linked driver.

  RETURNS

    Pointer to a string containing the driver identification.

\*---------------------------------------------------------------------------*/
void
);
#endif


BldRc bldConstructDriver(
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Tells the driver the communications parameters to be used with the
    board(s). This function only needs to be called in DOS. In Windows 95
    and Windows NT the information is available to the driver from the
    Windows Registry.

    The function verifies that supplied IRQ numbers and I/O addresses
    are valid and that the boards are installed (or seem to be installed,
    rather). It also installs the Interrupt Service Routines (ISRs), but
    not until disabling the IRQ from all LI's on all boards.

  PLATFORMS

    DOS ONLY

  RETURNS

    BLD_SUCCESS

    BLD_WRONG_CONTEXT
    BLD_INVALID_BOARD_TYPE
    BLD_INVALID_BOARD_NO
    BLD_INVALID_IRQ_NO
    BLD_INVALID_ADDR
    BLD_NO_BOARD

\*---------------------------------------------------------------------------*/
  short  boardType,     /* Type of the board used. Boardtype for Balder-2S is
                        .. 122 and for Balder-8S is 128.                     */
  unsigned long boardBaseAddr, /* I/0-base address wrapped on the board. Base
                        .. address for Balder boards is valid if:
                        .. (0x000 < Addr <= 0x400) && (Addr % 0x20) == 0     */
  short  noOfBoards,    /* Number of Boards installed.                       */
  short  irq            /* IRQ to be used. All installed Balder-8S boards
                        .. can share the same IRQ.                           */
);



BldRc bldInitDriver(
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Connects to the Driver and initializes the Driver internal data
    structures. In Windows 95 and Windows NT this function must be
    called before any other functions are called.  In DOS this function
    needs to be called after bldConstructDriver().

  PLATFORMS

    ALL

  RETURNS

    BLD_SUCCESS               // OK

\*---------------------------------------------------------------------------*/
  void
);



BldRc bldDestructDriver(
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    This function must be called before the application is exited.
    Cleans up and clears the driver after use. Uninstalls the
    Interrupt Service Routines installed by the bldConstructDriver().

  RETURNS

    BLD_SUCCESS

    BLD_WRONG_CONTEXT

\*---------------------------------------------------------------------------*/
void
);

BldRc bldResetDriver(
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Resets the Driver software. Clears the receive and transmit memory
    buffers.

  RETURNS

    BLD_SUCCESS

  SEE ALSO

    BldResetLi()

\*---------------------------------------------------------------------------*/
  void
);


BldRc bldConfigureLi(
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Sets up the Line Interface (ISAC-S) chip in either Terminal (TE)
    or Network (NT) mode.

  RETURNS

    BLD_INVALID_LI_NO
    BLD_NO_BOARD
    BLD_INVALID_MODE
    ..                  If none of the above then code from bldResetLi()

\*---------------------------------------------------------------------------*/
  short liNo,           /* Number of the Li chip to be configured            */
  LiMode liMode         /* Mode to be configured to, BLD_TE or BLD_LT_S      */
);


BldRc bldConfigureMonitor(
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Sets up a pair of Line Interfaces to operate in monitor mode. The
    first Li will listen as a TE. The second Li will listen as an NT.

  RETURNS

    BLD_INVALID_LI_NO
    BLD_NO_BOARD
    ..                  If none of the above then code from bldResetLi()

\*---------------------------------------------------------------------------*/
  short teLiNo,         /* Number of the Li to listen as a TE                */
  short ntLiNo          /* Number of the Li to listen as a NT                */
);



#ifndef B8_LITE
BldRc bldCreateDspProgImage(
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Converts a DSP Application file in COFF format (*.out or *.obj) to
    the Balder-8 DSP Image format (used by bldConfigureDspImage()).

  RETURNS

    BLD_SUCCESS on success

    BLD_DATA_TOO_LARGE if the buffer supplied by the host is too small to
                       fit the entire DSP Application Image.

\*---------------------------------------------------------------------------*/
  char *appFile,                    /* Application file to be converted to an image    */
  char *entryPoint,                 /* Label for the entry point of the application     */
                                    /* Usually "_c_int00" for a C-compiled application  */
  unsigned short *imageBuf,         /* Image Buffer provided by the host */
  unsigned short imageBufSize,      /* Size of imageBuf (in elements) */
  long *imageEntryAddr,             /* Entry address; output from this function */
  unsigned short *noOfImageElements /* No of image elements returned in imageBuf */
);
#endif // !B8_LITE



#ifndef B8_LITE
BldRc bldConfigureDspImage(
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Loads and starts a DSP application program. The DSP application
    program must be in the Balder-8 DSP Image format (created by a
    call to bldCreateDspProgImage(). This function must be called
    prior to using any of the API functions which operate on the
    b-channels.


  RETURNS

    BLD_SUCCESS on success

    BLD_DSP_COMM_FAILURE
    BLD_DSP_NO_KERNEL_RESP
    BLD_FOPEN_ERROR
    BLD_DSP_INVALID_ENTRYPT

\*---------------------------------------------------------------------------*/
  short dspNo,                 /* Number of the DSP to be used      */
  unsigned short *progImage,   /* Program image */
  unsigned short progImageLen, /* Number of elements in progImage */
  long progEntryAddr,          /* Program Entry Address */
  BldLawType law               /* U-law or A-law companding law */
);
#endif // !B8_LITE


#ifndef B8_LITE
BldRc bldConfigureDsp(
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Loads and starts a DSP application program. The application file
    must be in TI COFF file format (*.out or *.obj). This function
    must be called prior to using any of the API functions which
    operate on the b-channels.


  RETURNS

    BLD_SUCCESS on success

    BLD_DSP_COMM_FAILURE
    BLD_DSP_NO_KERNEL_RESP
    BLD_FOPEN_ERROR
    BLD_DSP_INVALID_ENTRYPT

\*---------------------------------------------------------------------------*/
  short dspNo,           /* Number of the DSP to be used                     */
  char *appfile,         /* Application file to be loaded                    */
  char *entryPoint,      /* Label for the entry point of the application     */
                         /* Usually "_c_int00" for a C-compiled application  */
  BldLawType law         /* u-law or a-law companding */
);
#endif // !B8_LITE


BldRc bldRestartDsp(
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Restarts an already loaded DSP application program.

  RETURNS

    BLD_SUCCESS on success


\*---------------------------------------------------------------------------*/
  short dspNo,           /* Number of the DSP to be used                     */
  BldLawType law         /* u-law or a-law companding */
);



BldRc bldRead(
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Read the next received message. Driver will automatically sort the
    received messages from different Li according to the reception
    time. If several messages are waiting to be read, this function
    will read and return the oldest one.

  RETURNS

    BLD_SUCCESS   Frame read successfully

    BLD_NO_FRAMES If no complete frames have been received and ready to
                  be read.

\*---------------------------------------------------------------------------*/
  unsigned char   fmBuf[],   /* Buffer into which the received message will
                             .. be written. NOTE: The buffer must be
                             .. allocated by the application.                */
  short           fmBufSize, /* Size of fmBuf[].                             */
  BldFrameHeader *fmHeader   /* Pointer to header structure that will be
                             .. fillied in by the function. NOTE: The struct
                             .. must be allocated by the application.        */
);


BldRc bldWriteLi(
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Send an ISDN message including layer 2. Use bldWriteIframeLi() to
    send I-frames when operating in auto mode.

  RETURNS

    BLD_SUCCESS      // Message accepted and is being sent

    BLD_TX_BUSY      // Previous message not yet sent completely
                     // Try again later

\*---------------------------------------------------------------------------*/
  short           liNo,      /* Number of the Line Interface to be used for
                             .. sending.                                     */
  unsigned char   fmBuf[],   /* Buffer containing the message to be send.    */
  short           fmLength   /* Length of the message to be sent.            */
);


#ifndef B8_LITE
BldRc bldWriteIframeLi(
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Send an ISDN I-frame message in auto mode. Layer2 excluded (added
    automatically by Balder).

  RETURNS

    BLD_SUCCESS      // Message accepted and is being sent

    BLD_TX_BUSY      // Previous message not yet sent completely
                     // Try again later

\*---------------------------------------------------------------------------*/
  short           liNo,
  unsigned char   fmBuf[],
  short           fmLength
);
#endif // !B8_LITE


BldRc bldByteOnBch(
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Start sending a constant byte on the specified B-channel and keep sending
    it until another B-channel function is called.

    A call to this function overrides any prior B-channel related
    calls (e.g. bldPhoneOn(), bldDtmfOnBch(), bldByteOnBch(),
    bldToneOnBch(), bldPlayBackBch(), bldSinewaveOnBch()).

    Note that this is the only B-channel function which does not use
    a companding law. The byte value is transmitted as specified (no
    companding is performed).


  RETURNS

    BLD_SUCCESS

  SEE ALSO

    bldByteOffBch()

\*---------------------------------------------------------------------------*/
  short liNo,                /* Number of the line interface to be used      */
  short bChannelNo,          /* Number of the B-channel to be used (1 or 2)  */
  unsigned char aByte        /* Byte to be send                              */
);


BldRc bldByteOffBch(
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Stop the sending of a constant byte value on the specified B-channel.

    Disconnect the Bchannel from the S/T by sending 0xFF.

  RETURNS
    BLD_SUCCESS

  SEE ALSO
    bldByteOnBch()
\*---------------------------------------------------------------------------*/
  short liNo,                /* Number of the line interface to be used      */
  short bChannelNo           /* Number of the B-channel to be used (1 or 2)  */
);


BldRc bldByteReadBch(
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Reads one byte from the specified B-channel and returns it in
    recByte.

  NOTE

    Returned byte is not companded.

  RETURNS

    BLD_SUCCESS

\*---------------------------------------------------------------------------*/
  short liNo,                /* Number of the line interface to be used      */
  short bChannelNo,          /* Number of the B-channel to be used (1 or 2)  */
  unsigned char *recByte     /* Received byte from the B-channel             */
);


BldRc bldGetStatusLi(
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Reads the status of a Line Interface.

  RETURNS

    BLD_L1_OK
    BLD_L1_DOWN
    BLD_UNDEFINED

\*---------------------------------------------------------------------------*/
  short liNo                 /* Number of the Line Interface to be read      */
);


BldRc bldGetStatusDtmf(
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    This function returns the most recent DTMF digit that was detected
    on a specified B-channel in the 'dtmfDigit' parameter. If there
    has been no detected DTMF tone since the last call to this
    function, then it returns BLD_NO_TONE.

  RETURNS

     BLD_TONE     DTMF tone has been detected (returned on dtmfDigit)

     BLD_NO_TONE  if no new DTMF tone has been detected since the last call

\*---------------------------------------------------------------------------*/
  short liNo,         /* Number of the line interface (DSP) to be used       */
  short bChannelNo,   /* Number of the B-channel to be used (1 or 2)         */
  char *dtmfDigit     /* Most recent DTMF digit detected (or \0 if none)     */
);


BldRc bldGetStatusSilence(
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    This function returns the status of the Silence Dectector for a
    specified B-channel.

  RETURNS

    BLD_NO_TONE  if the Silence Detector has detected silence

    BLD_TONE     if the Silence Detector has detected non-silence

\*---------------------------------------------------------------------------*/
  short liNo,         /* Number of the line interface (DSP) to be used       */
  short bChannelNo    /* Number of the B-channel to be used (1 or 2)         */
);


BldRc bldResetLi(
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Performs a reset on the HDLC for the Line Interface.

  RETURNS

    BLD_SUCCESS

    BLD_L1_DOWN
    BLD_RESET_FAILED
    BLD_NO_CLOCKS
    BLD_BAD_CHIP

  SEE ALSO

    bldResetDriver()

\*---------------------------------------------------------------------------*/
  short liNo            /* Number of the Line Interface to reset             */
);


BldRc bldActivate(
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Issues a Layer 1 Activate Request.

  RETURNS

    BLD_SUCCESS

    BLD_TIMEOUT
    BLD_INVALID_LI_NO
    BLD_ALREADY_ACTIVATED

\*---------------------------------------------------------------------------*/
  short liNo,           /* Number of the Line Interface to activate          */
  long timeout          /* Time in milliseconds to wait for the layer 1 to
                        .. become active                                     */
);


BldRc bldDeactivate(
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Issues a Layer 1 Deactivate Request. Only valid if the Li is
    configured in LT_S (NT) mode.

  RETURNS

    BLD_SUCCESS

    BLD_WRONG_CONTEXT

\*---------------------------------------------------------------------------*/
  short liNo            /* Number of the Line Interface to deactivate        */
);


#ifndef B8_LITE
char *bldGetErrMsg(
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Converts a Bld-function return code into a string describing the
    error. Returns a pointer to a static string owned by the function.

  RETURNS

    A pointer to the test string describing the error code in a general
    fashion. Can be used for quick and dirty solutions when the return
    code is not analyzed properly by the application but something needs
    to be printed.

\*---------------------------------------------------------------------------*/
  BldRc errCode         /* Error code to be converted                        */
);
#endif // !B8_LITE


BldRc bldPhoneOn(
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Connects one of the two Analog Interface Circuits (AIC0 or AIC1)
    to one of the two B-channels in one of the 8 DSPs. Note that only one
    AIC can be connected to one DSP at any time.

  RETURNS

    BLD_SUCCESS

    BLD_INVALID_LI_NO
    BLD_DSP_NOT_CONFIGURED
    BLD_INVALID_B_CHANNEL


\*---------------------------------------------------------------------------*/
  short liNo,               /* Number of the line interface to be used          */
  short bChannelNo,         /* Number of the BCH to be used                     */
  short aicNo,              /* Number of the Analog Interface Circuit be used   */
  BldAnalogInSrc inSrc,     /* Specifies what the Analog Interface Circuit should be used to send B-channel data to the Line interface */
  BldAnalogOutSrc outSrc,   /* Specifies what to send to the speaker */
  BldDigGainType digGain,   /* Digital gain of generated tones */
  BldSpkrGainType spkrGain  /* Gain or attenuation to speaker */
);



BldRc bldPhoneOff(
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    If the indicated DSP (liNo) has an Analog Interface Circuit
    connected then this function will disconnect the AIC from this
    DSP.

  RETURNS

    BLD_SUCCESS

    BLD_INVALID_LI_NO
    BLD_INVALID_B_CHANNEL

\*---------------------------------------------------------------------------*/
  short liNo            /* Number of the line interface to be used           */
);




/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Dials a phone number using DTMF tones on a B-channel. The time between
    tones can be specified in the 'silence' parameter, and the duration of
    each tone is determined by the value of the 'duration' parameter.

    A call to this function overrides any prior B-channel related
    calls (e.g. bldPhoneOn(), bldDtmfOnBch(), bldByteOnBch(),
    bldToneOnBch(), bldPlayBackBch(), bldSinewaveOnBch()).

  RETURNS

    BLD_SUCCESS

    BLD_INVALID_LI_NO
    BLD_DSP_NOT_CONFIGURED
    BLD_INVALID_B_CHANNEL


\*---------------------------------------------------------------------------*/
BldRc bldDtmfOnBch(
  short liNo,               /* Number of the line interface to be used       */
  short bChannelNo,         /* Number of the BCH to be used                  */
  char *phoneNumber,        /* Phone number to dial                          */
  short duration,           /* Duration of DTMF tones in 12.75 ms increments ..
                            ..  (0 for infinity)                             */
  short silence,            /* Silence time between of DTMF tones in 12.75 ms..
                            .. increments                                    */
  BldDigGainType digGain    /* Digital gain of generated tones               */
);


/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Configures the DSP to receive HDLC frames on the specified
    B-channel. The received frames are buffered in the driver, and can
    be retrieved by calling bldRead().

    The HDLC frame retrieved by bldRead will include the 16-bit Frame
    Check Sequence (FCS) if this functions is called with the
    parameter includeFcs=1.

    The FCS will always be checked according to the CCITT Q.921 CRC
    Generator Polynomial (x^16 + x^12 + x^5 + 1) no matter if
    includeFcs is set or not. If the frame passes the CRC then the
    frame will have fmStatus==BLD_FEC_OK in fmHeader (just like HDLC
    frames received on the D-channel).

    A call to this function overrides any prior B-channel related
    calls (e.g. bldPhoneOn(), bldDtmfOnBch(), bldByteOnBch(),
    bldToneOnBch(), bldPlayBackBch(), bldSinewaveOnBch()).

\*---------------------------------------------------------------------------*/
BldRc bldHdlcOnBch(
  short dspNo,             /* Number of the line interface (DSP) to be used  */
  short bChannelNo,        /* Number of the BCH to be used                   */
  short includeFcs,        /* =1 to include the 16-bit FCS in the HDLC frame */
  BldFrameFillType filler  /* Either BLD_FFT_ALL_ONES or BLD_FFT_FLAGS       */
);


/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Connects or disconnects the 100-ohm terminating resistor for a
    specified Line Interface.

  RETURNS

    BLD_SUCCESS

    BLD_INVALID_LI_NO

\*---------------------------------------------------------------------------*/
BldRc bldTermLi(
  short liNo,                /* Number of the line interface to be used      */
  short connect              /* 1=CONNECT, 0=DISCONNECT                      */
);


/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Returns the maximum number of bytes which each of the FIFO have
    contained since the FIFOs were initialized.

  RETURNS

    BLD_SUCCESS

\*---------------------------------------------------------------------------*/
BldRc bldFifoUsage(
  short liNo,                /* Number of the line interface to be used      */
  unsigned short *rxHdlcFifo,
  unsigned short *rxInfoFifo,
  unsigned short *txHdlcFifo
);


#ifndef B8_LITE
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Digital volume control for the generated sound on a B-channel. It
    sets the digital gain.

  RETURNS

    BLD_SUCCESS

    BLD_INVALID_LI_NO
    BLD_DSP_NOT_CONFIGURED
    BLD_INVALID_B_CHANNEL


\*---------------------------------------------------------------------------*/
BldRc bldDigGainOnBch(
  short liNo,                /* Number of the line interface to be used      */
  short bChannelNo,          /* Number of the B-channel to be used (1 or 2)  */
  BldDigGainType digGain     /* Digital gain of generated tone               */
);
#endif // !B8_LITE


/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Configures the Silence Detector. A silence is detected when a
    certain number of received samples have a combined energy level of
    less than the specified threshold.

  RETURNS

    BLD_SUCCESS

    BLD_INVALID_LI_NO
    BLD_DSP_NOT_CONFIGURED
    BLD_INVALID_B_CHANNEL


\*---------------------------------------------------------------------------*/
BldRc bldConfigureSilenceDetector(
  short liNo,                /* Number of the line interface (DSP) to be
                             .. used                                         */
  short bChannelNo,          /* Number of the B-channel to be used (1 or 2)  */
  short enableDetector,      /* 1 = Silence Detector Enabled, 0 = Silence
                             .. Detector Disabled                            */
  short silenceDuration,     /* Time (in 12.75ms increments) that silence
                             .. must be present to be reported               */
  short thresHold            /* A low thresHold stipulates that very little
                             .. noise will be accepted for silence to be
                             .. reported; A higher thresHold accepts more
                             .. more noise in the silence                    */
);


/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Configures the DTMF Detector.

    The received digits can either be stored in the FIFO and be
    extracted with a call bldRead(), or if storeInFifo == 0 then only
    the most recently received digit can be retrieved with a call to
    bldGetStatusDtmf().

  RETURNS

    BLD_SUCCESS

    BLD_INVALID_LI_NO
    BLD_DSP_NOT_CONFIGURED
    BLD_INVALID_B_CHANNEL


\*---------------------------------------------------------------------------*/
BldRc bldConfigureDtmfDetector(
  short liNo,               /* Number of the line interface (DSP) to be used */
  short bChannelNo,         /* Number of the B-channel to be used (1 or 2)   */
  short detectorEnable,     /* 1 = Enable DTMF detector, 0 = Disable it      */
  short storeInFifo         /* 1 = Store received digits in Fifo, 0 = Don't
                            .. store. Only remember the last received digit  */
);


/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Configures the FSK Detector.

    The received octets are placed in the FIFO
    (BldFrameType==BLD_FM_FSK1 for Bch1 and BldFrameType==BLD_FM_FSK1
    for Bch2) and be retrieved by calling bldRead().

  RETURNS

    BLD_SUCCESS

    BLD_INVALID_LI_NO
    BLD_INVALID_B_CHANNEL

\*---------------------------------------------------------------------------*/
BldRc bldConfigureFskDetector(
  short liNo,               /* Number of the line interface (DSP) to be used */
  short bChannelNo,         /* Number of the B-channel to be used (1 or 2)   */
  short detectorEnable      /* 1 = Enable FSK detector, 0 = Disable it      */
);


/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Start generating a desired sound on a specified B-channel.

    A call to this function overrides any prior B-channel related
    calls (e.g. bldPhoneOn(), bldDtmfOnBch(), bldByteOnBch(),
    bldToneOnBch(), bldPlayBackBch(), bldSinewaveOnBch()).

  RETURNS

    BLD_SUCCESS
    BLD_INVALID_LI_NO
    BLD_DSP_NOT_CONFIGURED
    BLD_INVALID_B_CHANNEL


\*---------------------------------------------------------------------------*/
BldRc bldToneOnBch(
  short liNo,                /* Number of the line interface to be used      */
  short bChannelNo,          /* Number of the B-channel to be used (1 or 2)  */
  BldToneType toneType,      /* Type of "tone" to be generated on the Bch    */
  BldDigGainType digGain     /* Digital gain of generated tone               */
);


#ifndef B8_LITE
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Start generating a custom sinewave on a specified B-channel.

    A call to this function overrides any prior B-channel related
    calls (e.g. bldPhoneOn(), bldDtmfOnBch(), bldByteOnBch(),
    bldPlayBackBch(), bldSinewaveOnBch()).

  RETURNS

    BLD_SUCCESS

    BLD_DSP_NOT_CONFIGURED
    BLD_INVALID_B_CHANNEL


\*---------------------------------------------------------------------------*/
BldRc bldSinewaveOnBch(
  short dspNo,                 /* Number of the line interface (DSP) to be used */
  short bChannelNo,            /* Number of the B-channel to be used (1 or 2)   */
  short coeff,                 /* Coefficient of sinewave (modifies frequency)  */
  short y1,                    /* Initial y(n-1) value (modifies amplitude)     */
  short y2,                    /* Initial y(n-2) value (modifies amplitude)     */
  BldDigGainType digGain       /* Digital gain of generated tone                */
);
#endif // !B8_LITE


/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Stops generating a sound on a specified B-channel. Disconnect the
    B-channel from the S/T bus by sending 0xFF. This function call has
    the same effect as a call to bldByteOffBch().


  RETURNS

    BLD_SUCCESS
    BLD_INVALID_LI_NO
    BLD_INVALID_B_CHANNEL

\*---------------------------------------------------------------------------*/
BldRc bldToneOffBch(
  short liNo,                  /* Number of the line interface to be used      */
  short bChannelNo             /* Number of the B-channel to be used (1 or 2)  */
);


#ifndef B8_DOS_DRV
BldRc bldRegisterCallback(
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Register a callback function in the application to be called by the
    driver when it receives a message or a status change.

  NOTES

   Only meaningful with Windows 95 and Windows NT drivers

  RETURNS

    BLD_SUCCESS                          // OK
    BLD_INVALID_CALLBACK_FUNCTION        // Invalid function pointer
    BLD_CALLBACK_ALREADY_SET             // Callback function can be set only once
    BLD_CANT_CREATE_CALLBACK_THREAD      // Creation of a new thread failed

\*---------------------------------------------------------------------------*/
  void (*notificationCallback)(void)    // function pointer to the callback function in
                                // the application to be called from the driver
);
#endif // !B8_DOS_DRV



/*---------------------------------------------------------------------------*\
  SYNOPSIS

    Initiates playback of previously recorded signed int samples on a
    B-channel.

    The samples must have a magnitude of -8192 < x < 8191 if
    companding==1. If companding==0 (raw data) then the samples (still
    signed int) must have a magnitude of 0 <= x <= 255.

    The sample buffer must be of a size which is a multiple of 102.

    The function bldPlaybackBch() is to be called to determine if
    it is time to send more samples to the driver.

    To terminate playback call bldPlaybackBchTerminate().

    A call to this function overrides any prior B-channel related
    calls (e.g. bldPhoneOn(), bldDtmfOnBch(), bldByteOnBch(),
    bldToneOnBch(), bldPlayBackBch(), bldSinewaveOnBch()).

  NOTE

    The plBuf supplied by the application does *not* need to be
    available to the driver between successive calls to
    bldPlaybackBch().

  RETURNS

    BLD_SUCCESS

    BLD_DSP_NOT_CONFIGURED
    BLD_INVALID_B_CHANNEL

    BLD_INVALID_SAMPLE_BUF_SIZE if sampleBufSize is not a multiple of 102

    BLD_TX_BUSY if the driver is still busy sending the data in the buffer
                given to him

\*---------------------------------------------------------------------------*/
BldRc bldPlaybackBchInitiate(
  short dspNo,             /* Number of the line interface (DSP) to be used */
  short bChannelNo,        /* Number of the BCH to be used                 */
  short plBuf[],           /* Buffer filled with samples to be played back */
  short plBufSize,         /* size of plBuf (ping+pong) (in shorts). Must be a multiple of 102 */
  short compandData,       /* 1 = if data is companded, 0 = if data is raw data */
  BldDigGainType digGain   /* Digital gain of the sent samples */
);


/*---------------------------------------------------------------------------*\
  SYNOPSIS

    To be called after the playback is initiated with
    bldPlaybackBchInitiate() to supply the driver with more playback
    samples.

  RETURNS

    BLD_SUCCESS if the the driver has completed sending the buffer
                given to him (in bldPlaybackBch()).

    BLD_DSP_NOT_CONFIGURED

    BLD_INVALID_B_CHANNEL

    BLD_TX_BUSY if the driver is still busy sending the data in the buffer
                given to him (in bldPlaybackBch()).

    BLD_PLAYBACK_NOT_INITITATED if bldPlaybackBchInitiate() has not been called
                prior to this function or if the host is too slow calling this
                function after the call to bldPlaybackBchInitiate()

\*---------------------------------------------------------------------------*/
BldRc bldPlaybackBch(
  short dspNo,             /* Number of the line interface (DSP) to be used */
  short bChannelNo,        /* Number of the B-channel to be used (1 or 2)  */
  short plBuf[]            /* Buffer filled with samples to be played back */
);


/*---------------------------------------------------------------------------*\
  SYNOPSIS

    To be called after the playback is completed.

  RETURNS

    BLD_SUCCESS

    BLD_DSP_NOT_CONFIGURED
    BLD_INVALID_B_CHANNEL

    BLD_TX_BUSY if the driver is still busy sending the data in the buffer
                given to him (in bldPlaybackBch()).

\*---------------------------------------------------------------------------*/
BldRc bldPlaybackBchTerminate(
  short dspNo,              /* Number of the line interface (DSP) to be used */
  short bChannelNo,         /* Number of the B-channel to be used (1 or 2)  */
  short force               /* =1 if we want to force playback to be terminated, =0 if we want to wait until the driver has filled the last buffer */
);


/*---------------------------------------------------------------------------*\
  SYNOPSIS

    Initiates recording of B-channel samples.

    The samples (signed int) will have a magnitude of -8192 < x < 8191
    if companding==1. If companding==0 (raw data) then the samples
    (still signed int) will have a magnitude of 0 <= x <= 255.

    The sample buffer must be of a size which is a multiple of 102.

    The function bldRecordBch() is to be called to determine if
    it is time to read recorded samples from the driver.

    To terminate recording call bldRecordBchTerminate().

  RETURNS

    BLD_SUCCESS if recording was sucessfully inititated. Call bldRecordBch() next.

    BLD_DSP_NOT_CONFIGURED

    BLD_INVALID_B_CHANNEL

    BLD_INVALID_SAMPLE_BUF_SIZE if sampleBufSize is not a multiple of 102

    BLD_REC_BUSY if the driver is still busy recording the data

\*---------------------------------------------------------------------------*/
BldRc bldRecordBchInitiate(
  short dspNo,            /* Number of the line interface (DSP) to be used */
  short bChannelNo,       /* Number of the BCH to be used                 */
  short recBufSize,       /* size of recBuf which will be used in bldRecordBch. Must be a multiple of 102 */
  short compandData       /* 1 = if data is companded, 0 = if data is raw data */
);


/*---------------------------------------------------------------------------*\
  SYNOPSIS

    To be called after the recording is initiated with
    bldRecordBchInitiate() to read the recorded samples from the
    driver.

  RETURNS

    BLD_SUCCESS if driver has filled recBuf with recorded samples.

    BLD_DSP_NOT_CONFIGURED

    BLD_INVALID_B_CHANNEL

    BLD_REC_BUSY if recorded data is not available yet. Try again.

    BLD_RECORD_NOT_INITITATED if bldRecordBchInitiate() has not been called
                prior to this function or if the host is too slow calling this
                function after the call to bldRecordBchInitiate()

\*---------------------------------------------------------------------------*/
BldRc bldRecordBch(
  short dspNo,             /* Number of the line interface (DSP) to be used */
  short bChannelNo,        /* Number of the B-channel to be used (1 or 2)  */
  short recBuf[]           /* Buffer to be filled with recorded samples */
);


/*---------------------------------------------------------------------------*\
  SYNOPSIS

    To be called after recording is completed.

  RETURNS

    BLD_SUCCESS

    BLD_DSP_NOT_CONFIGURED

    BLD_INVALID_B_CHANNEL

\*---------------------------------------------------------------------------*/
BldRc bldRecordBchTerminate(
  short dspNo,              /* Number of the line interface (DSP) to be used */
  short bChannelNo          /* Number of the B-channel to be used (1 or 2)  */
);



#ifndef B8_LITE
/*---------------------------------------------------------------------------*\

  SYNOPSIS

    Sends a command to the DSP. This function is only used for debugging
    the DSP code.

  RETURNS

     BLD_SUCCESS

\*---------------------------------------------------------------------------*/
BldRc bldDspCmd(
  short liNo,                  /* Number of the line interface to be used      */
  short dspCmd                 /* DSP Command to be sent    */
);
#endif // !B8_LITE



#ifndef B8_LITE
/*--------------------------------------------------------------------------*\

  SYNOPSIS

    Converts a Status code to a string.

  RETURNS

    A pointer to a string providing a brief description of the status code.
    The string is owned by the driver function.

\*--------------------------------------------------------------------------*/
const char *bldStatus2Str(
  BldStatusCodeType code
);
#endif // !B8_LITE


#ifdef __cplusplus
  } // extern "C"
#endif

/* $History: BALDER.H $
 * 
 * *****************  Version 12  *****************
 * User: Hanz         Date: 5/09/98    Time: 12:04p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Inc
 * Added bldConfigureMonitor() to configure a pair of Lis in monitor mode.
 * One Li will be configured in BLD_TE mode and the other in BLD_MON mode.
 * BLD_MON mode is basically BLD_TE mode, except that he does not know
 * about the line status. He will get the line status from his liPartner
 * (the BLD_TE guy).
 *
 * *****************  Version 11  *****************
 * User: Jari         Date: 4/25/98    Time: 12:27p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Inc
 * Made boardBaseAddr unsigned long.
 *
 * *****************  Version 10  *****************
 * User: Hanz         Date: 3/26/98    Time: 5:42p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Inc
 * Added bldDspCreateAppImg() and bldConfigureDspImage() to faciliate
 * faster loading of the DSPs.
 *
 * *****************  Version 9  *****************
 * User: Hanz         Date: 3/18/98    Time: 12:17p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Inc
 * Inplemented the recording functions with Ping Pong buffers (in the same
 * way as playback).
 *
 * *****************  Version 8  *****************
 * User: Hanz         Date: 3/18/98    Time: 7:42a
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Inc
 * Changed the plackback function so that he uses a ping-pong buffer for
 * data transferred from the application to the driver. This works in the
 * BC driver. B8TST runs OK.
 *
 * *****************  Version 7  *****************
 * User: Hanz         Date: 3/16/98    Time: 5:33p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Inc
 * Added support for Aborted HDLC frames on the B-channel. Also added the
 * FCS check. B8TST runs OK. HDLC B-channel frames only tested with a few
 * samples files (3ff, abort, and 3msg).
 *
 * *****************  Version 6  *****************
 * User: Hanz         Date: 3/15/98    Time: 10:44p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Inc
 * Added support for HDLC reception on the B-channels. Not quite
 * complemeted. Need to add two more HDLC fifos in the host driver, and
 * add  the FCS check.
 *
 * *****************  Version 5  *****************
 * User: Hanz         Date: 3/08/98    Time: 8:48p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Inc
 * Changed parameters to bldPhoneOn() so that the microphone and speaker
 * are treated as two separate entities. Started to implement a handset
 * "monitor" functionality where the handset speaker gets both the DSP
 * generated data and the received data from ISAC. This functionality does
 * not work yet. Also implemented separate speaker gain for the handset
 * speaker. Also implemented a "raw" playback/recording option.
 *
 * *****************  Version 4  *****************
 * User: Hanz         Date: 2/21/98    Time: 4:04p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Inc
 * Renamed bldResetDsp() to bldRestartDsp(). Removed the law parameter
 * from the api function and added the law parameter to bldResetDsp() and
 * bldConfigureDsp(). I.e. we only set the companding law when we start
 * the DSP program.
 *
 * *****************  Version 3  *****************
 * User: Hanz         Date: 2/03/98    Time: 11:44p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Msc51/Inc
 * Changed some comments.
 *
 * *****************  Version 2  *****************
 * User: Hanz         Date: 1/26/98    Time: 3:20p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Msc51/Inc
 * Added more ifdef's for B8_LITE. We are now down to 27046 bytes of code
 * size for the Lite driver.
 *
 * *****************  Version 1  *****************
 * User: Hanz         Date: 1/25/98    Time: 10:13p
 * Created in $/Products/Bld8-20/Design/Drv/Dos/Msc51/Inc
 *
 * *****************  Version 22  *****************
 * User: Hanz         Date: 1/25/98    Time: 9:39p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Msc51
 *  Added enablePlayback parameter to bldPlaybackBch().
 *
 * *****************  Version 21  *****************
 * User: Hanz         Date: 1/24/98    Time: 11:25p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Msc51
 * Implemented bldRecordBch(). Now works for both B-channels.
 *
 * *****************  Version 20  *****************
 * User: Hanz         Date: 1/24/98    Time: 8:23p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Msc51
 * Implemented a new version of playback where the ISR is sending the data
 * from a buffer supplied by the application.
 *
 * *****************  Version 19  *****************
 * User: Hanz         Date: 1/23/98    Time: 9:01p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Msc51
 * Fixed compiler warnings in MSC51. Added bldStatus2Str() function. Added
 * new fmType BLD_FM_STAT.
 *
 * *****************  Version 18  *****************
 * User: Hanz         Date: 1/19/98    Time: 8:24p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc
 * Removed bldGetStatusFsk().
 *
 * *****************  Version 17  *****************
 * User: Hanz         Date: 1/16/98    Time: 3:26p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc
 * Implemented new command to allow the host to reset the DSP (make him
 * start over from _c_int00).
 *
 * *****************  Version 16  *****************
 * User: Hanz         Date: 12/26/97   Time: 8:27p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc Sohl Fix
 * Added bldDspCmd().
 *
 * *****************  Version 15  *****************
 * User: Neil         Date: 12/11/97   Time: 5:24p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc
 * Added configureFskDetector( ) and getStatusFsk( ). This function only
 * works in DOS now and will need to be added to 95/NT.
 *
 * *****************  Version 14  *****************
 * User: Jari         Date: 11/11/97   Time: 12:46p
 * Updated in $/Products/Bld8-20/Design/Drv/W95/VxD
 *
 * *****************  Version 13  *****************
 * User: Neil         Date: 11/07/97   Time: 11:24a
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc
 * Added apiSinewaveOnBch( ) function which allows the host to set the
 * sinewave frequency and amplitude.
 *
 * *****************  Version 12  *****************
 * User: Lme          Date: 11/05/97   Time: 2:47p
 * Updated in $/Products/Bld8-20/Design/Drv/NT/Kmd
 * Implemented and alpha tested Event Notification.
 *
 * *****************  Version 11  *****************
 * User: Jari         Date: 10/23/97   Time: 4:41p
 * Updated in $/Products/Bld8-20/Design/Drv/W95/VxD
 * Converted int's to short's to make porting between 16 and 32-bit
 * platforms easier.
 *
 * *****************  Version 10  *****************
 * User: Jari         Date: 10/21/97   Time: 1:43p
 * Updated in $/Products/Bld8-20/Design/Drv/W95/VxD
 *
 * *****************  Version 9  *****************
 * User: Jari         Date: 10/18/97   Time: 12:26p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc
 * bldConstructDriver() split into bldConstructDriver() and
 * bldInitDriver()
 *
 * *****************  Version 8  *****************
 * User: Jari         Date: 10/18/97   Time: 11:47a
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc
 * Changed constructDriver to accept only one IRQ value.
 *
 * *****************  Version 7  *****************
 * User: Jari         Date: 10/15/97   Time: 6:34p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc
 * Split into balder.h and blddef.h
 *
 */

#endif   /* BALDER_H */
